#pragma once
#include "DataType.h"

class MeshLoader
{
protected:
	std::vector<Node> nodes;
	std::vector<Finite_Element> finite_elems;
	std::vector<Boundary_Finite_Element> boundary;
public:
	virtual ~MeshLoader() = default;
	virtual void LoadMesh(const std::string&) = 0;
	const std::vector<Node>& Get_node() const;
	const std::vector<Finite_Element>& Get_finite_elems() const;
	const std::vector<Boundary_Finite_Element>& Get_boundary() const;

	std::vector<int> Get_elems_by_ID(int p_node_id1, int p_node_id2, int p_node_id3) const;
	std::vector<int> Get_elems_by_edge(int p_node_id1, int p_node_id2) const;
	std::vector<int> Get_elems_by_material(int p_material_id) const;

	std::set<int> Get_nodes_by_surface(int p_surface_id) const;

	std::vector<int> Get_boundary_by_surface(int p_surface_id) const;

	void Insert_new_nodes();

	std::vector<std::set<int>> Get_adjacent_nodes()const;

private:
	std::array<double, 3> Get_middle_of_edge(const Edge&);
	Node Get_middle_node(const Edge&);
	Node& Get_node_by_ID(int);
	std::array<double, 3> Get_coors_node_by_ID(int);
};
